@@ -8,11 +8,12 @@ Core logic of model fitter. Does not include plots.
 """
 
 from SBstoat.namedTimeseries import NamedTimeseries, TIME, mkNamedTimeseries
+from SBstoat.observationSynthesizer import  \
+      ObservationSynthesizerRandomizedResiduals
 from SBstoat.logs import Logger
 import SBstoat.timeseriesPlotter as tp
 from SBstoat import namedTimeseries
 from SBstoat import rpickle
-from SBstoat import _helpers
 
 import collections
 import copy
@@ -28,9 +29,12 @@ import typing
 PARAMETER_LOWER_BOUND = 0
 PARAMETER_UPPER_BOUND = 10
 #  Minimizer methods
-METHOD_BOTH = "both"
 METHOD_DIFFERENTIAL_EVOLUTION = "differential_evolution"
-METHOD_LEASTSQ = "leastsqr"
+METHOD_BOTH = "both"
+METHOD_LEASTSQ = "leastsq"
+METHOD_FITTER_DEFAULTS = [METHOD_DIFFERENTIAL_EVOLUTION, METHOD_LEASTSQ]
+METHOD_BOOTSTRAP_DEFAULTS = [METHOD_LEASTSQ,
+      METHOD_DIFFERENTIAL_EVOLUTION, METHOD_LEASTSQ]
 MAX_CHISQ_MULT = 5
 PERCENTILES = [2.5, 97.55]  # Percentile for confidence limits
 INDENTATION = "  "
@@ -52,22 +56,30 @@ class ParameterSpecification(object):
 class ModelFitterCore(rpickle.RPickler):
 
     def __init__(self, modelSpecification, observedData,
-                 parametersToFit=None,
-                 selectedColumns=None,
-                 fitterMethods=[METHOD_BOTH],
-                 bootstrapMethods=[METHOD_LEASTSQ, METHOD_BOTH],
-                 parameterLowerBound=PARAMETER_LOWER_BOUND,
-                 parameterUpperBound=PARAMETER_UPPER_BOUND,
-                 parameterDct={},
-                 fittedDataTransformDct={},
-                 logger=Logger(),
-                 isPlot=True,
-                 _loggerPrefix="",
-                 **bootstrapKwargs
-                 ):
+          parametersToFit=None,
+          selectedColumns=None,
+          fitterMethods=METHOD_FITTER_DEFAULTS,
+          numFitRepeat=1,
+          bootstrapMethods=METHOD_BOOTSTRAP_DEFAULTS,
+          parameterLowerBound=PARAMETER_LOWER_BOUND,
+          parameterUpperBound=PARAMETER_UPPER_BOUND,
+          parameterDct={},
+          fittedDataTransformDct={},
+          logger=Logger(),
+          isPlot=True,
+          _loggerPrefix="",
+          # The following must be kept in sync with ModelFitterBootstrap.bootstrap
+          numIteration:int=10, 
+          reportInterval:int=1000,
+          synthesizerClass=ObservationSynthesizerRandomizedResiduals,
+          maxProcess:int=None,
+          serializePath:str=None,
+          ):
         """
+        Constructs estimates of parameter values. 
+    
         Parameters
-        ---------
+        ----------
         modelSpecification: ExtendedRoadRunner/str
             roadrunner model or antimony model
         observedData: NamedTimeseries/str
@@ -84,7 +96,7 @@ class ModelFitterCore(rpickle.RPickler):
             upper bound for the fitting parameters
         parameterDct: dict
             key: parameter name
-            value: ParameterSpecification
+            value: triple - (lowerVange, startingValue, upperRange)
         fittedDataTransformDct: dict
             key: column in selectedColumns
             value: function of the data in selectedColumns;
@@ -93,14 +105,26 @@ class ModelFitterCore(rpickle.RPickler):
         logger: Logger
         fitterMethods: str/list-str
             method used for minimization in fitModel
+        numFitRepeat: int
+            number of times fitting is repeated for a method
         bootstrapMethods: str/list-str
             method used for minimization in bootstrap
-        bootstrapKwargs: dict
-            Parameters used in bootstrap
+        numIteration: number of bootstrap iterations
+        reportInterval: number of iterations between progress reports
+        synthesizerClass: object that synthesizes new observations
+            Must subclass ObservationSynthesizer
+        maxProcess: Maximum number of processes to use. Default: numCPU
+        serializePath: Where to serialize the fitter after bootstrap
 
         Usage
         -----
-        f = ModelFitter(roadrunnerModel, "observed.csv", ['k1', 'k2'])
+        parameterDct = {
+            "k1": (1, 5, 10),  # name of parameter: low value, initial, high
+            "k2": (2, 3, 6)}
+        ftter = ModelFitter(roadrunnerModel, "observed.csv",
+            parameterDct=parameterDct)
+        fitter.fitModel()  # Do the fit
+        fitter.bootstrap()  # Estimate parameter variance with bootstrap
         """
         if modelSpecification is not None:
             # Not the default constructor
@@ -109,8 +133,14 @@ class ModelFitterCore(rpickle.RPickler):
             self.parametersToFit = parametersToFit
             self.lowerBound = parameterLowerBound
             self.upperBound = parameterUpperBound
-            self.bootstrapKwargs = bootstrapKwargs
+            self.bootstrapKwargs = dict(
+                  numIteration=numIteration,
+                  reportInterval=reportInterval,
+                  maxProcess=maxProcess,
+                  serializePath=serializePath,
+                  )
             self.parameterDct = self._updateParameterDct(parameterDct)
+            self._numFitRepeat = numFitRepeat
             if self.parametersToFit is None:
                 self.parametersToFit = [p for p in self.parameterDct.keys()]
             self.observedTS = observedData
@@ -127,7 +157,10 @@ class ModelFitterCore(rpickle.RPickler):
             # Other internal state
             self._fitterMethods = fitterMethods
             if isinstance(self._fitterMethods, str):
-                self._fitterMethods = [self._fitterMethods]
+                if self._fitterMethods == METHOD_BOTH:
+                    self._fitterMethods = METHOD_FITTER_DEFAULTS
+                else:
+                    self._fitterMethods = [self._fitterMethods]
             self._bootstrapMethods = bootstrapMethods
             if isinstance(self._bootstrapMethods, str):
                 self._bootstrapMethods = [self._bootstrapMethods]
@@ -483,8 +516,11 @@ class ModelFitterCore(rpickle.RPickler):
         ##^
         self.logger.endBlock(guid)
         #
-        if np.isnan(np.sum(residualsArr)):
-            import pdb; pdb.set_trace()
+        # Used for detailed debugging
+        if False:
+            self.logger.details("_residuals/std(residuals): %f" % 
+                  np.std(residualsArr))
+            self.logger.details("_residuals/params: %s" % str(params))
         return residualsArr
 
     def fitModel(self, params:lmfit.Parameters=None,
@@ -503,6 +539,8 @@ class ModelFitterCore(rpickle.RPickler):
         -------
         f.fitModel()
         """
+        ParameterDescriptor = collections.namedtuple("ParameterDescriptor",
+              "params method std minimizer minimizerResult")
         block = Logger.join(self._loggerPrefix, "fitModel")
         guid = self.logger.startBlock(block)
         self._initializeRoadrunnerModel()
@@ -512,54 +550,42 @@ class ModelFitterCore(rpickle.RPickler):
         else:
             if params is None:
                 params = self.mkParams()
-            residuals_DE = self.observedTS.flatten()
-            residuals_LS = residuals_DE
-            params_DE = None
-            params_LS = None
-            # Fit the model to the data
-            # Use two algorithms:
-            #   Global differential evolution to get us close to minimum
-            #   A local Levenberg-Marquardt to getsus to the minimum
-            isMinimized = False
+            # Fit the model to the data using one or more methods.
+            # Choose the result with the lowest residual standard deviation
+            paramDct = {}
             for method in self._fitterMethods:           
-                if method in [METHOD_BOTH, METHOD_DIFFERENTIAL_EVOLUTION]:
+                for _ in range(self._numFitRepeat):
                     minimizer = lmfit.Minimizer(self._residuals, params,
                           max_nfev=max_nfev)
                     try:
-                        self.minimizerResult = minimizer.minimize(
-                              method=METHOD_DIFFERENTIAL_EVOLUTION,
-                              max_nfev=max_nfev)
-                        params_DE = self.minimizerResult.params
-                        residuals_DE = self._residuals(params=params_DE)
-                        isMinimized = True
+                        minimizerResult = minimizer.minimize(
+                              method=method, max_nfev=max_nfev)
                     except Exception as excp:
-                        self.logger.error(
-                              "Minimizer failed using differential evolution", excp)
-                        method = METHOD_LEASTSQ
-                if method in [METHOD_BOTH, METHOD_LEASTSQ]:
-                    minimizer = lmfit.Minimizer(self._residuals, params,
-                          max_nfev=max_nfev)
-                    self.minimizerResult = minimizer.minimize(
-                          method=METHOD_LEASTSQ,
-                          max_nfev=max_nfev)
-                    params_LS = self.minimizerResult.params
-                    residuals_LS = self._residuals(params=params_LS)
-                    isMinimized = True
-                if isMinimized:
-                    break
-            if not isMinimized:
-                raise ValueError("Minimizer failed")
-            if np.std(residuals_DE) <= np.std(residuals_LS):
-                self.params = params_DE
-            else:
-                if params_LS is not None:
-                    self.params = params_LS
-                else:
-                    self.params = params_DE
-            self.minimizer = minimizer
-            if not self.minimizer.success:
+                        msg = "Error minimizing for method: %s" % method
+                        self.logger.error(msg, excp)
+                        continue
+                    params = minimizerResult.params
+                    std = np.std(self._residuals(params))
+                    if method in paramDct.keys():
+                        if std >= paramDct[method].std:
+                            continue
+                    paramDct[method] = ParameterDescriptor(
+                          params=params.copy(),
+                          method=method,
+                          std=std,
+                          minimizer=minimizer,
+                          minimizerResult=minimizerResult,
+                          )
+            if len(paramDct) == 0:
                 msg = "*** Minimizer failed for this model and data."
                 raise ValueError(msg)
+            # Select the result that has the smallest residuals
+            sortedMethods = sorted(paramDct.keys(),
+                  key=lambda m: paramDct[m].std)
+            bestMethod = sortedMethods[0]
+            self.params = paramDct[bestMethod].params
+            self.minimizer= paramDct[bestMethod].minimizer
+            self.minimizerResult = paramDct[bestMethod].minimizerResult
         # Ensure that residualsTS and fittedTS match the parameters
         self.updateFittedAndResiduals(params=self.params)
         self.logger.endBlock(guid)
@@ -679,4 +705,3 @@ class ModelFitterCore(rpickle.RPickler):
             fitter = rpickle.load(fd)
         fitter._initializeRoadrunnerModel()
         return fitter
-
